#include "CreatorMap.h"
#include "OEMSolver.h"
#include "ITL_Solver.h"
#include "ITL_Preconditioner.h"
#include "NonLinSolver.h"
#include "MatrixVector.h"
#include "SystemVector.h"
#include "Estimator.h"
#include "RecoveryEstimator.h"
#include "ResidualEstimator.h"
#include "LeafData.h"
#include "SurfaceRegion_ED.h"
#include "DOFMatrix.h"
#include "ElementRegion_ED.h"
#include "UmfPackSolver.h"

namespace AMDiS {

  template<>
  void CreatorMap<OEMSolver>::addDefaultCreators()
  {
    OEMSolverCreator *creator;

    creator = NEW CGSolver::Creator;
    addCreator("cg", creator);

    creator = NEW BiCGSolver::Creator;
    addCreator("bicg", creator);

    creator = NEW BiCGStab::Creator;
    addCreator("bicgstab", creator);

    creator = NEW BiCGStab2::Creator;
    addCreator("bicgstab2", creator);

    creator = NEW BiCGStabEll::Creator;
    addCreator("bicgstab_ell", creator);

#ifdef HAVE_UMFPACK
    creator = NEW UmfPackSolver::Creator;
    addCreator("umfpack", creator);
#endif
  }

  template<>
  void CreatorMap<ITL_BasePreconditioner>::addDefaultCreators()
  {
    CreatorInterface<ITL_BasePreconditioner> *creator;

    creator =  NEW DiagonalPreconditioner::Creator;
    addCreator("diag", creator);

    creator = NEW ILUPreconditioner::Creator;
    addCreator("ilu", creator);

    creator = NEW ICPreconditioner::Creator;
    addCreator("ic", creator);

    creator =  NEW IdentityPreconditioner::Creator;
    addCreator("no", creator);
  }

#if 0
  template<>
  void CreatorMap<NonLinSolver<DOFVector<double> > >::addDefaultCreators()
  {
    NonLinSolverCreator<DOFVector<double> > *creator;

    creator = NEW Newton<DOFVector<double> >::Creator;
    addCreator("newton", creator);

    creator = NEW NewtonS<DOFVector<double> >::Creator;
    addCreator("newton_fs", creator);
  }
#endif


#if 0
  template<>
  void CreatorMap<NonLinSolver<SystemVector> >::addDefaultCreators()
  {
    NonLinSolverCreator<SystemVector> *creator;

    creator = NEW Newton<SystemVector>::Creator;
    addCreator("newton", creator);

    creator = NEW NewtonS<SystemVector>::Creator;
    addCreator("newton_fs", creator);
  }
#endif

  template<>
  void CreatorMap<Estimator>::addDefaultCreators()
  {
    EstimatorCreator *creator;

    creator = NEW ResidualEstimator::Creator;
    addCreator("residual", creator);

    creator = NEW RecoveryEstimator::Creator;
    addCreator("recovery", creator);
  }

  template<>
  void CreatorMap<ElementData>::addDefaultCreators()
  {
    CreatorInterface<ElementData> *creator;

    creator = NEW LeafDataEstimatable::Creator;
    addCreator("LeafDataEstimatable", creator);

    creator = NEW LeafDataEstimatableVec::Creator;
    addCreator("LeafDataEstimatableVec", creator);

    creator = NEW LeafDataCoarsenable::Creator;
    addCreator("LeafDataCoarsenable", creator);

    creator = NEW LeafDataCoarsenableVec::Creator;
    addCreator("LeafDataCoarsenableVec", creator);

    creator = NEW LeafDataPeriodic::Creator;
    addCreator("LeafDataPeriodic", creator);

    creator = NEW SurfaceRegion_ED::Creator;
    addCreator("SurfaceRegion_ED", creator);

    creator = NEW ElementRegion_ED::Creator;
    addCreator("ElementRegion_ED", creator);
  }

}
